@@ -1,4 +1,5 @@
-# $Id: test_dryrun.rb 40251 2013-04-11 17:00:49Z nagachika $
+# frozen_string_literal: true
+# $Id: test_dryrun.rb 62738 2018-03-13 06:29:02Z nobu $
 
 require 'fileutils'
 require 'test/unit'
@@ -7,7 +8,7 @@ require_relative 'visibility_tests'
 class TestFileUtilsDryRun < Test::Unit::TestCase
 
   include FileUtils::DryRun
-  include TestFileUtils::Visibility
+  include TestFileUtilsInc::Visibility
 
   def setup
     super
