@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'test/unit'
 
 class TestAssignment < Test::Unit::TestCase
@@ -102,7 +103,7 @@ class TestAssignment < Test::Unit::TestCase
   end
 
   def test_assign_private_self
-    bug9907 = '[ruby-core:62949] [Bug #9907]'
+    bug11096 = '[ruby-core:68984] [Bug #11096]'
 
     o = Object.new
     class << o
@@ -127,18 +128,18 @@ class TestAssignment < Test::Unit::TestCase
       assert_equal(1, o.instance_eval {self[0] = 1})
     }
 
-    assert_nothing_raised(NoMethodError, bug9907) {
-      assert_equal(43, o.instance_eval {self.foo += 1})
+    assert_raise(NoMethodError, bug11096) {
+      o.instance_eval {self.foo += 1}
     }
-    assert_nothing_raised(NoMethodError, bug9907) {
-      assert_equal(1, o.instance_eval {self.foo &&= 1})
+    assert_raise(NoMethodError, bug11096) {
+      o.instance_eval {self.foo &&= 1}
     }
 
-    assert_nothing_raised(NoMethodError, bug9907) {
-      assert_equal(43, o.instance_eval {self[0] += 1})
+    assert_raise(NoMethodError, bug11096) {
+      o.instance_eval {self[0] += 1}
     }
-    assert_nothing_raised(NoMethodError, bug9907) {
-      assert_equal(1, o.instance_eval {self[0] &&= 1})
+    assert_raise(NoMethodError, bug11096) {
+      o.instance_eval {self[0] &&= 1}
     }
   end
 
@@ -479,11 +480,10 @@ class TestAssignment < Test::Unit::TestCase
     assert_equal 1, a
     assert_equal [2, 3], b
 
-    # not supported yet
-    #a, *b, c = 1, 2, 3, 4
-    #assert_equal 1, a
-    #assert_equal [2,3], b
-    #assert_equal 4, c
+    a, *b, c = 1, 2, 3, 4
+    assert_equal 1, a
+    assert_equal [2,3], b
+    assert_equal 4, c
 
     a = 1, 2
     assert_equal [1, 2], a
@@ -551,6 +551,11 @@ class TestAssignment < Test::Unit::TestCase
     a, b = Base::A, Base::B
     assert_equal [3,4], [a,b]
   end
+
+  def test_massign_in_cond
+    result = eval("if (a, b = MyObj.new); [a, b]; end", nil, __FILE__, __LINE__)
+    assert_equal [[1,2],[3,4]], result
+  end
 end
 
 require_relative 'sentence'
@@ -765,4 +770,14 @@ class TestAssignmentGen < Test::Unit::TestCase
     h[*k], = ["ok", "ng"]
     assert_equal("ok", h[:key], bug11970)
   end
+
+  def test_chainged_assign_command
+    all_assertions do |a|
+      asgn = %w'= +='
+      asgn.product(asgn) do |a1, a2|
+        stmt = "a #{a1} b #{a2} raise 'x'"
+        a.for(stmt) {assert_valid_syntax(stmt)}
+      end
+    end
+  end
 end
