@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'test/unit'
 require 'securerandom'
 require 'tempfile'
@@ -69,45 +70,14 @@ if false
   end
 end
 
-  def test_s_random_bytes_without_openssl
-    begin
-      require 'openssl'
-    rescue LoadError
-      return
-    end
-    begin
-      load_path = $LOAD_PATH.dup
-      loaded_features = $LOADED_FEATURES.dup
-      openssl = Object.instance_eval { remove_const(:OpenSSL) }
-
-      remove_feature('securerandom.rb')
-      remove_feature('openssl.rb')
-      Dir.mktmpdir do |dir|
-        open(File.join(dir, 'openssl.rb'), 'w') { |f|
-          f << 'raise LoadError'
-        }
-        $LOAD_PATH.unshift(dir)
-        v = $VERBOSE
-        begin
-          $VERBOSE = false
-          require 'securerandom'
-        ensure
-          $VERBOSE = v
-        end
-        test_s_random_bytes
-      end
-    ensure
-      $LOADED_FEATURES.replace(loaded_features)
-      $LOAD_PATH.replace(load_path)
-      Object.const_set(:OpenSSL, openssl)
-    end
-  end
-
   def test_s_hex
-    assert_equal(16 * 2, @it.hex.size)
+    s = @it.hex
+    assert_equal(16 * 2, s.size)
+    assert_match(/\A\h+\z/, s)
     33.times do |idx|
-      assert_equal(idx * 2, @it.hex(idx).size)
-      assert_equal(idx, @it.hex(idx).gsub(/(..)/) { [$1].pack('H*') }.size)
+      s = @it.hex(idx)
+      assert_equal(idx * 2, s.size)
+      assert_match(/\A\h*\z/, s)
     end
   end
 
@@ -128,23 +98,20 @@ end
       assert_not_match(safe, @it.urlsafe_base64(idx))
     end
     # base64 can include unsafe byte
-    10001.times do |idx|
-      return if safe =~ @it.base64(idx)
-    end
-    flunk
+    assert((0..10000).any? {|idx| safe =~ @it.base64(idx)}, "None of base64(0..10000) is url-safe")
   end
 
   def test_s_random_number_float
     101.times do
       v = @it.random_number
-      assert(0.0 <= v && v < 1.0)
+      assert_in_range(0.0...1.0, v)
     end
   end
 
   def test_s_random_number_float_by_zero
     101.times do
       v = @it.random_number(0)
-      assert(0.0 <= v && v < 1.0)
+      assert_in_range(0.0...1.0, v)
     end
   end
 
@@ -152,15 +119,35 @@ end
     101.times do |idx|
       next if idx.zero?
       v = @it.random_number(idx)
-      assert(0 <= v && v < idx)
+      assert_in_range(0...idx, v)
     end
   end
 
+  def test_s_random_number_not_default
+    msg = "SecureRandom#random_number should not be affected by srand"
+    seed = srand(0)
+    x = @it.random_number(1000)
+    10.times do|i|
+      srand(0)
+      return unless @it.random_number(1000) == x
+    end
+    srand(0)
+    assert_not_equal(x, @it.random_number(1000), msg)
+  ensure
+    srand(seed) if seed
+  end
+
   def test_uuid
     uuid = @it.uuid
     assert_equal(36, uuid.size)
-    uuid.unpack('a8xa4xa4xa4xa12').each do |e|
-      assert_match(/^[0-9a-f]+$/, e)
+    assert_match(/\A\h{8}-\h{4}-\h{4}-\h{4}-\h{12}\z/, uuid)
+  end
+
+  def test_alphanumeric
+    65.times do |n|
+      an = @it.alphanumeric(n)
+      assert_match(/\A[0-9a-zA-Z]*\z/, an)
+      assert_equal(n, an.length)
     end
   end
 
@@ -182,4 +169,19 @@ end
     }
   end
 
+  def assert_in_range(range, result, mesg = nil)
+    assert(range.cover?(result), message(mesg) {"Expected #{result} to be in #{range}"})
+  end
+
+  def test_with_openssl
+    begin
+      require 'openssl'
+    rescue LoadError
+      return
+    end
+    assert_equal(Encoding::ASCII_8BIT, @it.send(:gen_random_openssl, 16).encoding)
+    65.times do |idx|
+      assert_equal(idx, @it.send(:gen_random_openssl, idx).size)
+    end
+  end
 end
