@@ -1,4 +1,5 @@
 # coding: us-ascii
+# frozen_string_literal: false
 
 begin
   require 'win32ole'
@@ -57,9 +58,10 @@ if defined?(WIN32OLE)
 
     def test_no_method_error
       exc = assert_raise(NoMethodError) {
-          @dict1.non_exist_method
+        @dict1.non_exist_method
       }
       assert_match(/non_exist_method/, exc.message)
+      assert_kind_of(WIN32OLE, exc.receiver)
     end
 
     def test_ole_methods
@@ -67,7 +69,12 @@ if defined?(WIN32OLE)
       mnames = methods.collect {|m|
         m.name
       }
-      assert(mnames.include?("Add"))
+      assert_include(mnames, 'Add')
+    end
+
+    def test_methods
+      methods = @dict1.methods
+      assert_include(methods, :Add)
     end
 
     def test_ole_func_methods
@@ -75,7 +82,7 @@ if defined?(WIN32OLE)
       mnames = methods.collect {|m|
         m.name
       }
-      assert(mnames.include?("Add"))
+      assert_include(mnames, 'Add')
     end
 
     def test_ole_put_methods
@@ -83,7 +90,7 @@ if defined?(WIN32OLE)
       mnames = methods.collect {|m|
         m.name
       }
-      assert(mnames.include?("CompareMode"))
+      assert_include(mnames, 'CompareMode')
     end
 
     def test_ole_get_methods
@@ -91,7 +98,7 @@ if defined?(WIN32OLE)
       mnames = methods.collect {|m|
         m.name
       }
-      assert(mnames.include?("Count"))
+      assert_include(mnames, 'Count')
     end
 
     def test_ole_mehtod_help
@@ -169,6 +176,39 @@ if defined?(WIN32OLE)
       }
     end
 
+    def test_s_new_exc_svr_tainted
+      th = Thread.start {
+        $SAFE = 1
+        svr = "Scripting.Dictionary"
+        svr.taint
+        Thread.current.report_on_exception = false
+        WIN32OLE.new(svr)
+      }
+      exc = assert_raise(SecurityError) {
+        th.join
+      }
+      assert_match(/insecure object creation - `Scripting.Dictionary'/, exc.message)
+    ensure
+      $SAFE = 0
+    end
+
+    def test_s_new_exc_host_tainted
+      th = Thread.start {
+        $SAFE = 1
+        svr = "Scripting.Dictionary"
+        host = "localhost"
+        host.taint
+        Thread.current.report_on_exception = false
+        WIN32OLE.new(svr, host)
+      }
+      exc = assert_raise(SecurityError) {
+        th.join
+      }
+      assert_match(/insecure object creation - `localhost'/, exc.message)
+    ensure
+      $SAFE = 0
+    end
+
     def test_s_new_DCOM
       rshell = WIN32OLE.new("Shell.Application")
       assert_instance_of(WIN32OLE, rshell)
@@ -194,6 +234,22 @@ if defined?(WIN32OLE)
       }
     end
 
+    def test_s_coonect_exc_tainted
+      th = Thread.start {
+        $SAFE = 1
+        svr = "winmgmts:"
+        svr.taint
+        Thread.current.report_on_exception = false
+        WIN32OLE.connect(svr)
+      }
+      exc = assert_raise(SecurityError) {
+        th.join
+      }
+      assert_match(/insecure connection - `winmgmts:'/, exc.message)
+    ensure
+      $SAFE = 0
+    end
+
     def test_invoke_accept_symbol_hash_key
       fso = WIN32OLE.new('Scripting.FileSystemObject')
       afolder = fso.getFolder(".")
@@ -228,7 +284,7 @@ if defined?(WIN32OLE)
     def test_invoke_hash_key_non_str_sym
       fso = WIN32OLE.new('Scripting.FileSystemObject')
       begin
-        bfolder = fso.getFolder({1 => "."})
+        fso.getFolder({1 => "."})
         assert(false)
       rescue TypeError
         assert(true)
@@ -270,7 +326,10 @@ if defined?(WIN32OLE)
     def test_ole_query_interface
       shell=WIN32OLE.new('Shell.Application')
       assert_raise(ArgumentError) {
-        shell2 = shell.ole_query_interface
+        shell.ole_query_interface
+      }
+      assert_raise(TypeError) {
+        shell.ole_query_interface(0x11223344)
       }
       shell2 = shell.ole_query_interface('{A4C6892C-3BA9-11D2-9DEA-00C04FB16162}')
       assert_instance_of(WIN32OLE, shell2)
