@@ -3,7 +3,7 @@
 
   thread_pthread.c -
 
-  $Author: normal $
+  $Author: naruse $
 
   Copyright (C) 2004-2007 Koichi Sasada
 
@@ -2206,4 +2206,23 @@ timer_pthread_fn(void *p)
     return 0;
 }
 #endif /* UBF_TIMER_PTHREAD */
+
+static VALUE
+ubf_caller(const void *ignore)
+{
+    rb_thread_sleep_forever();
+
+    return Qfalse;
+}
+
+/*
+ * Called if and only if one thread is running, and
+ * the unblock function is NOT async-signal-safe
+ * This assumes USE_THREAD_CACHE is true for performance reasons
+ */
+static VALUE
+rb_thread_start_unblock_thread(void)
+{
+    return rb_thread_create(ubf_caller, 0);
+}
 #endif /* THREAD_SYSTEM_DEPENDENT_IMPLEMENTATION */
